@@ -12,13 +12,9 @@ import logging
 
 from . import ipsub
 
-try:
-    import cStringIO
-except ImportError:
-    import StringIO as cStringIO
-
 from chorde.clients import CacheMissError
 from chorde.clients import inproc
+from chorde.clients.memcached import JSONASCIIEncoder
 
 P2P_HWM = 10
 INPROC_HWM = 1 # Just for wakeup signals
@@ -61,9 +57,8 @@ def _psufix(x):
 
 HASHES = {
     int : lambda x : hash(x) & 0xFFFFFFFF, # hash(x:int) = x, but must be 64/32-bit compatible
-    long : lambda x : hash(x) & 0xFFFFFFFF, # must be 64/32-bit compatible
-    str : _psufix, # prefix+suffix is ok for most
-    unicode : lambda x : _psufix(x.encode("utf8")),
+    bytes : _psufix, # prefix+suffix is ok for most
+    str : lambda x : _psufix(x.encode("utf8")),
     list : len,
     set : len,
     dict : len,
@@ -89,7 +84,7 @@ def _bound_weak_callback(self, f):
 
 def _mkwaiter(ctx, socktype, prefix):
     waiter = ctx.socket(zmq.PAIR)
-    for _ in xrange(5):
+    for _ in range(5):
         waiter_id = "inproc://%s%x.%x" % (prefix, id(waiter),random.randint(0,1<<30))
         try:
             waiter.bind(waiter_id)
@@ -125,7 +120,7 @@ def _noop(*p, **kw):
 class CoherenceManager(object):
     def __init__(self, namespace, private, shared, ipsub_,
             p2p_pub_bindhosts = DEFAULT_P2P_BINDHOSTS,
-            encoding = 'pyobj',
+            encoding = b'pyobj',
             synchronous = False,
             quick_refresh = False,
             stable_hash = stable_hash,
@@ -183,6 +178,11 @@ class CoherenceManager(object):
             logger = logging.getLogger('chorde.mq.coherence')
         self.logger = logger
 
+        if isinstance(encoding, str):
+            encoding = encoding.encode("utf8")
+        if isinstance(namespace, str):
+            namespace = namespace.encode("utf8")
+
         self.private = private
         self.shared = shared
         self.ipsub = ipsub_
@@ -192,7 +192,7 @@ class CoherenceManager(object):
         self.quick_refresh = quick_refresh
         self.stable_hash = stable_hash
         self.encoding = encoding
-        self._txid = itertools.cycle(xrange(0x7FFFFFFF))
+        self._txid = itertools.cycle(range(0x7FFFFFFF))
 
         # Key -> hash
         self.pending = dict()
@@ -214,17 +214,17 @@ class CoherenceManager(object):
         self.p2p_pub_binds = [ipsub_.identity] # default contact is ipsub identity
 
         # Active broadcasts
-        self.delprefix = namespace + '|c|del|'
-        self.delackprefix = namespace + '|c|delack|'
+        self.delprefix = namespace + b'|c|del|'
+        self.delackprefix = namespace + b'|c|delack|'
 
         # Listener -> Broker requests
-        self.pendprefix = namespace + '|c|pend|'
-        self.pendqprefix = namespace + '|c|pendq|'
-        self.doneprefix = namespace + '|c|done|'
-        self.abortprefix = namespace + '|c|abrt|'
+        self.pendprefix = namespace + b'|c|pend|'
+        self.pendqprefix = namespace + b'|c|pendq|'
+        self.doneprefix = namespace + b'|c|done|'
+        self.abortprefix = namespace + b'|c|abrt|'
 
         # Broker -> Listener requests
-        self.listpendqprefix = namespace + '|c|listpendq|'
+        self.listpendqprefix = namespace + b'|c|listpendq|'
 
         # FSM state event lister handles, not registered yet
         self.encoded_pending = None
@@ -265,7 +265,7 @@ class CoherenceManager(object):
         Generate a new transaction id. No two reads will be the same... often.
         """
         # Iterator is atomic, no need for locks
-        return self._txid.next()
+        return next(self._txid)
 
     @_swallow_connrefused(_noop)
     def fire_deletion(self, key, timeout = None):
@@ -273,7 +273,7 @@ class CoherenceManager(object):
         waiter = self.waiter(self, txid) # subscribe before publishing, or we'll miss it
         if key is CLEAR:
             # Cannot use any other encoding for this
-            encoding = 'pyobj'
+            encoding = b'pyobj'
         else:
             encoding = self.encoding
         self.ipsub.publish_encode(self.delprefix, encoding, (txid, key), timeout = timeout)
@@ -501,7 +501,7 @@ class CoherenceManager(object):
                     return True
             ipsub_.listen(self.namespace, ipsub.EVENT_UPDATE_ACKNOWLEDGED, signaler)
             ipsub_.publish(self.pendqprefix, req)
-            for i in xrange(3):
+            for i in range(3):
                 if waiter.poll(timeout/4):
                     break
                 elif expired():
@@ -573,7 +573,7 @@ class CoherenceManager(object):
                 rv = (rv, now, self.p2p_pub_binds)
         if lock and rv is None:
             self.group_pending[key] = (txid, now, contact)
-        return ipsub.BrokerReply(json.dumps(rv))
+        return ipsub.BrokerReply(json.dumps(rv, cls=JSONASCIIEncoder).encode("ascii"))
 
     @_weak_callback
     def _on_list_pending_query(self, prefix, event, payload):
@@ -586,7 +586,7 @@ class CoherenceManager(object):
     def _on_pending(self, prefix, event, payload):
         if self.ipsub.is_broker:
             txid, keys, contact = payload
-            self.group_pending.update(itertools.izip(
+            self.group_pending.update(zip(
                 keys, itertools.repeat((txid,time.time(),contact),len(keys))))
             return True
 
@@ -674,8 +674,8 @@ class CoherenceManager(object):
         if keys:
             if txid is None:
                 txid = self.txid
-            first_key = iter(keys).next()
-            self.ipsub.publish_encode(self.doneprefix+str(self.stable_hash(first_key)), self.encoding,
+            first_key = next(iter(keys))
+            self.ipsub.publish_encode(self.doneprefix+bytes(self.stable_hash(first_key)), self.encoding,
                 (txid, keys, self.p2p_pub_binds),
                 timeout = timeout)
         return NoopWaiter()
@@ -685,8 +685,8 @@ class CoherenceManager(object):
         if keys:
             if txid is None:
                 txid = self.txid
-            first_key = iter(keys).next()
-            self.ipsub.publish_encode(self.abortprefix+str(self.stable_hash(first_key)), self.encoding,
+            first_key = next(iter(keys))
+            self.ipsub.publish_encode(self.abortprefix+bytes(self.stable_hash(first_key)), self.encoding,
                 (txid, keys, self.p2p_pub_binds),
                 timeout = timeout)
         return NoopWaiter()
@@ -719,7 +719,7 @@ class CoherenceManager(object):
             return True
 
         ipsub_ = self.ipsub
-        keysuffix = str(self.stable_hash(key))
+        keysuffix = bytes(self.stable_hash(key))
         doneprefix = self.doneprefix+keysuffix
         abortprefix = self.abortprefix+keysuffix
 
@@ -735,7 +735,7 @@ class CoherenceManager(object):
                     contact_cell[:] = contact
                     signaler = ctx.socket(zmq.PAIR)
                     signaler.connect(waiter_id)
-                    signaler.send("")
+                    signaler.send(b"")
                     signaler.close()
                     return False
                 else:
