@@ -5,16 +5,10 @@ import logging
 import json
 import os
 from collections import defaultdict
+from base64 import b64encode
 
-try:
-    import cPickle
-except ImportError:
-    import pickle as cPickle  # lint:ok
-
-try:
-    import cStringIO
-except ImportError:
-    import StringIO as cStringIO  # lint:ok
+import pickle
+import io
 
 __ALL__ = (
     'EVENT_INCOMING_UPDATE',
@@ -38,9 +32,9 @@ __ALL__ = (
     'BaseIPSub',
 )
 
-FRAME_HEARTBEAT = "__HeyDude__"
-FRAME_UPDATE_OK = "OK"
-FRAME_UPDATE_DROPPED = "DROP"
+FRAME_HEARTBEAT = b"__HeyDude__"
+FRAME_UPDATE_OK = b"OK"
+FRAME_UPDATE_DROPPED = b"DROP"
 FRAME_VALID_UPDATE_REPLIES = (FRAME_UPDATE_OK, FRAME_UPDATE_DROPPED)
 
 # EVENT KEY                         message payload format
@@ -70,8 +64,8 @@ EVENT_NAMES = {
 
 IDENTITY_EVENTS = (EVENT_INCOMING_UPDATE,)
 
-FRAME_UPDATE_OK = "OK"
-FRAME_UPDATE_DROPPED = "DROP"
+FRAME_UPDATE_OK = b"OK"
+FRAME_UPDATE_DROPPED = b"DROP"
 FRAME_VALID_UPDATE_REPLIES = (FRAME_UPDATE_OK, FRAME_UPDATE_DROPPED)
 
 EVENT_FOR_REPLY = {
@@ -85,21 +79,21 @@ IDLE_PERIOD = 0.5
 TIC_PERIOD = 1.0
 
 ENCODINGS = {
-    'json' : lambda x : json.dumps(x, separators=(',',':')),
-    'bytes' : lambda x : x,
-    'utf8' : lambda x : x.encode('utf8') if isinstance(x, unicode) else x,
+    b'json' : lambda x : json.dumps(x, separators=(',',':')).encode("ascii"),
+    b'bytes' : lambda x : x,
+    b'utf8' : lambda x : x.encode('utf8') if isinstance(x, str) else x,
 }
 
 DECODINGS = {
-    'json' : json.loads,
-    'bytes' : lambda x : x,
-    'utf8' : lambda x : x.decode('utf8'),
+    b'json' : json.loads,
+    b'bytes' : lambda x : x,
+    b'utf8' : lambda x : x.decode('utf8'),
 }
 
 STREAMDECODINGS = {
-    'json' : json.load,
-    'bytes' : lambda x : x.read(),
-    'utf8' : lambda x : x.read().decode('utf8'),
+    b'json' : json.load,
+    b'bytes' : lambda x : x.read(),
+    b'utf8' : lambda x : x.read().decode('utf8'),
 }
 
 class BrokerReply(object):
@@ -117,10 +111,10 @@ class BaseIPSub(object):
         self.last_idle = time.time()
         self.last_tic = time.time()
 
-        self.identity = "%x-%x-%s" % (
+        self.identity = b"%x-%x-%s" % (
             os.getpid(),
             id(self),
-            os.urandom(8).encode("base64").strip('\t =\n'),
+            b64encode(os.urandom(8)).strip(b'\t =\n'),
         )
 
     def _idle(self):
@@ -150,7 +144,7 @@ class BaseIPSub(object):
         """
         try:
             self.last_tic = time.time() - TIC_PERIOD
-            self._pushsocket().send("tic")
+            self._pushsocket().send(b"tic")
         except zmq.ZMQError:
             # Shit happens, probably not connected
             pass
@@ -179,6 +173,8 @@ class BaseIPSub(object):
         once (per instance, not function name). They should return fast,
         or the I/O thread may stall.
         """
+        if isinstance(prefix, str):
+            prefix = prefix.encode("utf8")
         self.listeners[event][prefix].add(callback)
         if event in IDENTITY_EVENTS:
             # Those are external, so we must subscribe
@@ -201,6 +197,8 @@ class BaseIPSub(object):
         """
         Removes the specified listener. See listen.
         """
+        if isinstance(prefix, str):
+            prefix = prefix.encode("utf8")
         if prefix in self.listeners[event]:
             try:
                 self.listeners[event][prefix].remove(callback)
@@ -252,7 +250,7 @@ class BaseIPSub(object):
                 prefix = update[0][:MAX_PREFIX]
             called = set()
             rrv = rv = None
-            for cb_prefix, callbacks in listeners.items():
+            for cb_prefix, callbacks in list(listeners.items()):
                 if prefix is None or prefix.startswith(cb_prefix):
                     byebye = set()
                     for callback in set(callbacks):
@@ -297,25 +295,27 @@ class BaseIPSub(object):
         """
         Publish the message using the 'json' encoding. See publish.
         """
-        self.publish(prefix, ['json',ENCODINGS['json'](payload)], timeout)
+        self.publish(prefix, [b'json',ENCODINGS[b'json'](payload)], timeout)
 
     def publish_pyobj(self, prefix, payload, timeout = None):
         """
         Publish the message using the 'pyobj' encoding. See publish.
         """
-        self.publish(prefix, ['pyobj',ENCODINGS['pyobj'](payload)], timeout)
+        self.publish(prefix, [b'pyobj',ENCODINGS[b'pyobj'](payload)], timeout)
 
     def publish_bytes(self, prefix, payload, timeout = None):
         """
         Publish the message using the 'bytes' encoding. See publish.
         """
-        self.publish(prefix, ['bytes',ENCODINGS['bytes'](payload)], timeout)
+        self.publish(prefix, [b'bytes',ENCODINGS[b'bytes'](payload)], timeout)
 
     def publish_unicode(self, prefix, payload, timeout = None):
         """
         Publish the message using the 'utf8' encoding. See publish.
         """
-        self.publish(prefix, ['utf8',ENCODINGS['utf8'](payload)], timeout)
+        self.publish(prefix, [b'utf8',ENCODINGS[b'utf8'](payload)], timeout)
+
+    publish_str = publish_unicode
 
     def publish_encode(self, prefix, encoding, payload, timeout = None):
         """
@@ -353,21 +353,21 @@ class BaseIPSub(object):
                 sPickles, they both work out-of-the-box.
         """
         def dumps(x):
-            io = cStringIO.StringIO()
-            p = pickler(io,2)
+            sio = io.BytesIO()
+            p = pickler(sio,2)
             p.dump(x)
-            return io.getvalue()
+            return sio.getvalue()
         def loads(x):
-            io = cStringIO.StringIO(x)
-            p = unpickler(io)
+            sio = io.BytesIO(x)
+            p = unpickler(sio)
             return p.load()
         def load(x):
             return unpickler(x).load()
-        BaseIPSub.register_encoding('pyobj', dumps, loads, load)
+        BaseIPSub.register_encoding(b'pyobj', dumps, loads, load)
 
     @staticmethod
     def register_default_pyobj():
-        BaseIPSub.register_pyobj(cPickle.Pickler, cPickle.Unpickler)
+        BaseIPSub.register_pyobj(pickle.Pickler, pickle.Unpickler)
 
     @staticmethod
     def encode_payload(encoding, payload):
